@@ -1,6 +1,6 @@
 from .base import BaseDevice, SwitchableDevice, PWMDevice
 from .button import Button
-from .fan import Fan
+from .fan import Fan, SM
 from .led import LED
 from .pwmled import PWMLED
 from .switch import Switch
