@@ -1,5 +1,7 @@
 import time
+
 import RPi.GPIO as gpio
+
 from .base import BaseDevice
 
 
@@ -10,24 +12,20 @@ class Button(BaseDevice):
     -:param debounce_time: Debounce time when checking button state (ms)
     -:param polarity: When you press the button, will the pin be connected to LOW or HIGH?
     """
-    def __init__(self, pin, debounce_time=200, polarity='LOW', **kwargs):
+    def __init__(self, pin, debounce_time=200, polarity=gpio.LOW, **kwargs):
         super().__init__(**kwargs)
         self.debounce_time = debounce_time / 1000
         self.polarity = polarity
         self.pin = pin
-        gpio.setup(pin, gpio.IN, pull_up_down={'HIGH': gpio.PUD_DOWN, 'LOW': gpio.PUD_UP}[polarity])
+        gpio.setup(pin, gpio.IN, pull_up_down={'0': gpio.PUD_UP, '1': gpio.PUD_DOWN}[str(polarity)])
 
     def is_pressed(self):
         """ True if the button is pressed """
-        def HIGH(pin):
-            if gpio.input(pin):
-                return True
-            return False
         def LOW(pin):
-            if not gpio.input(pin):
-                return True
-            return False
-        test = {'HIGH': HIGH, 'LOW': LOW}[self.polarity]
+            return bool(not gpio.input(pin))
+        def HIGH(pin):
+            return bool(gpio.input(pin))
+        test = {'0': LOW, '1': HIGH}[str(self.polarity)]
 
         if test(self.pin):
             time.sleep(self.debounce_time)
